#include <stdio.h>
#include <sys/types.h>
#include <unistd.h>
#include <fcntl.h>
#include <signal.h>
#include <sys/socket.h>

#include <tcl.h>

#include "message.h"

int init_comm(Comm *comm)
{
   if ((socketpair(AF_UNIX, SOCK_STREAM, 0, comm->parent_to_child) == 0)
       && (socketpair(AF_UNIX, SOCK_STREAM, 0, comm->child_to_parent) == 0))
   {

      fprintf(stderr, "Opened communication channels\n");

      fprintf(stderr, "Opened passageway to parent\n");

      return(TCL_OK);
   }
   else
   {
      fprintf(stderr, "Could not open the socketpair\n");
      return(TCL_ERROR);
   }
}

void init_parent_comm(Comm *comm)
{
   close((comm->child_to_parent)[SEND]);

   if (fcntl((comm->child_to_parent)[RECV], F_SETFL, FASYNC) == -1)
      fprintf(stderr, "Error - while setting recv channel to async\n");

   if (fcntl((comm->child_to_parent)[RECV], F_SETOWN, getpid()) == -1)
      fprintf(stderr, "Error - while setting recv signal\n");

   close((comm->parent_to_child)[RECV]);
}

void init_child_comm(Comm *comm)
{
   close((comm->parent_to_child)[SEND]);

   if (fcntl((comm->parent_to_child)[RECV], F_SETFL, FASYNC) == -1)
      fprintf(stderr, "Error - while setting recv channel to async\n");

   if (fcntl((comm->parent_to_child)[RECV], F_SETOWN, getpid()) == -1)
      fprintf(stderr, "Error - while setting recv signal\n");

   close((comm->child_to_parent)[RECV]);
}

void send_command_to_child(Comm *comm, char message)
{
   write((comm->parent_to_child)[SEND], &message, 1);
}

void recv_command_from_parent(Comm *comm, char *message)
{
   read((comm->parent_to_child)[RECV], message, 1);
}

void send_command_to_parent(Comm *comm, char message)
{
   write((comm->child_to_parent)[SEND], &message, 1);
}

void recv_command_from_child(Comm *comm, char *message)
{
   read((comm->child_to_parent)[RECV], message, 1);
}

void send_data_to_child(Comm *comm, char *Buffer, int Buffer_len)
{
   write((comm->parent_to_child)[SEND], Buffer, Buffer_len);
}

void recv_data_from_parent(Comm *comm, char *Buffer, int *Buffer_len)
{
   *Buffer_len = read((comm->parent_to_child)[RECV], Buffer, *Buffer_len);
}

#ifdef TEST
main()
{
   Comm comm;
   int  pid;
   char Command = ' ';
   char Buffer[256];
   int  Buffer_len;

   if (init_comm(&comm) == 0)
   {
      fprintf(stderr, "error opening socketpair\n");
      exit(1);
   }

   pid = fork();

   if (pid == 0)
   {
      /* child */
      recv_command_from_parent(&comm, &Command);
      printf("receive command form parent %c\n", Command);

      Command = '2';
      send_command_to_parent(&comm, &Command);

      Buffer_len = 256;
      recv_data_from_parent(&comm, Buffer, &Buffer_len);
      printf("receive data: %s\n", Buffer);
   }
   else
   {
      /* parent */
      Command = '1';
      send_command_to_child(&comm, Command);

      recv_command_from_child(&comm, &Command);
      printf("receive command form child %c\n", Command);

      strcpy(Buffer, "Matrix data\0");
      send_data_to_child(&comm, Buffer, strlen(Buffer) + 1);
   }
}
#endif
